#!/usr/local/bin/perl
#
# Copyright 1999 World Wide Web Consortium,
# (Massachusetts Institute of Technology, Institut
# National de Recherche en Informatique et en
# Automatique, Keio University). All Rights Reserved.
# This program is distributed under the W3C's
# Intellectual Property License. This program is
# distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See W3C License
# http://www.w3.org/Consortium/Legal/ for more details.
#
##############################################################################
#
# slidemaker tool
# split a all.htm into slide*.htm
#
# Stephan Montigaud - stephan@w3.org
# created 970601
# modified by Pierre Fillault
# check the documentation at http://www.w3.org/Talks/YYMMsub/
#
# modified 19990505 Bert Bos: ALT text of prev/next arrows is now
# "prev"/"next" rather than the title of the prev/next slide; looks better
# in lynx.
# modified 2005/10/17 Irene Vatton: Generate only one slide kit
#
# version: 4.14 - 19990719
# $Id: w3cburst.pl,v 1.4 2005/10/18 07:23:20 vatton Exp $
#
# NOTE: all the anchors in this tool contain the extension
# indeed this package has to work outside the environment of a web server
# Users need to be able to display their presentation using file:\\
# which requires link to 'real' files

##############################################################################
## default values of variables
##

## default DOCTYPE added on the slides
$doctype = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN"
    "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">';

## name of raw HTML file containing the slides
$all = 'all.htm';

## table of content built from all.htm - also first page of the presentation
## this is only the basename as we need to generate one toc for each style sheets
## the main toc will not bear any more so the server can understand a request for '/'
## the next ones will bear a number corresponding to the slide index
$overview = 'Overview';

## name of the file containing the parameters of the presentation
$infos = 'infos.txt';

## link to the logo printed on all the slides
$logoLink = 'http://www.inria.fr/';

## default location of the logo - works when slidemaker is used as a package
$logoFile = '../Icons/Inria_logo_small.gif';

## if slidemaker is used/ online, then the location of the logo is likely to be different
## than the default
if (! -f $logoFile) {
	# if the logo is not found, then we're likely to find it
	# at /Talks/Icons/ for W3C AFS users
	$logoFile = '/Talks/Icons/Inria_logo_small.gif';
}

## alternate representation of the logo
$logoAlt = 'Inria ';

## default values set to none
$logoLink2 = '';     # link to a potential second reference
$logoFile2 = '';     # location of a second logo
$logoAlt2  = '';     # alternate representation of the second logo

## navigation icons used in the slides
$left  = '../Icons/left.png';
$right = '../Icons/right.png';
$top   = '../Icons/up.png';
$toc   = '../Icons/toc.png';

## for online use on the W3C site
if (! -d '../Icons') {
	## the icons will be present at this default location
	$left  = '/Talks/Icons/left.png';
	$right = '/Talks/Icons/right.png';
	$top   = '/Talks/Icons/up.png';
	$toc   = '/Talks/Icons/toc.png';
}

## default accesskeys for navigation icons used in the slides
$prevKey  = 'P';	# accesskey for previous slide
$nextKey  = 'N';	# accesskey for next slide
$tocKey   = 'C';	# accesskey for table of contents

## default author name
$author = 'WAM Staff';

## default presentation title
$talkTitle = 'WAM Talk';

## standard style sheets 
if (!$cssFile) {
  $cssFile = '../Tools/inriatalk.css';
  if (! -d '../Tools') {
    ## online location for W3C use
  $cssFile = '/Talks/Tools/inriatalk.css';
  }
}

## default charset use in meta tag http-equiv
$charset = 'ISO-8859-1';

## default language setting is English 
$baselang = 'en-US';
$body = '<body>';

## end of default values for the presentation
##############################################################################

##############################################################################
## reading user input from $infos
##
@PARAM = @ARGV; # we keep this for backward compatibility with an old version
                # of the slidemaker tool
                #when the parameters were in Makefile or make.bat

# read parameters from infos.txt and put them in @PARAM
if (open(INFOS, $infos)) {
    print STDOUT "--- Reading parameters file $infos ---\n";
    local(@file,$counter);
    $counter = 0;
    @file = <INFOS>;
    @PARAM = ();
    do {
	    if ($file[0] =~ /^[^#\n\r]/) {
	      $file[0] =~ s/\n//;    # remove UNIX \n 
	      $file[0] =~ s/\r//;    # remove WINDOWS \r    
	      $file[0] =~ s/ *= */=/;
	      $PARAM[$counter++] = $file[0];
	      print "$file[0]\n";
	    }
    } while (shift(@file));
}
## @PARAM is now a table with the user preferences for his presentation

## process arguments
## each preset variable is now re-attributed using the user preferences
foreach (@PARAM) {
    @_ = split(/ *= */);
    $cmd="\$$_[0] = \'$_[1]\';";
    if (length $_[1] != 0) { 
	    eval($cmd);
    }
}

## put both authors together
if ($author2) {
    $author = $author." &amp; ".$author2;
}

##############################################################################
## read the raw html presentation
##

## copy file in memory
my $sep = $/;
$/ = undef;
if (!open(ALL, $all)) {
    print "Error: Cannot open file: $all\n";
    exit 0;
}
my $buf = <ALL>;
close(ALL);
$/ = $sep;

## Remove comments from the raw presentation
## they do not need to show up on the slides
$buf =~ s/<!--.*?-->//sgo;

## the slidemaker tool assumes that each slide is self contained between 2 sets of h1 tags
## if not it will generate a rather weird output
## split using <h1...> and </h1...> as separator (ignores attributes!)
## h1 or H1 can be used
@table = split(/<\/?[hH]1[^>]*>/, $buf);

## compute the total number of slides
$total = $#table / 2;
if ($#table % 2 != 0) {
	$total = ($#table +1)/2;
}

##
## raw presentation has been read successfully
##############################################################################

##############################################################################
## processing the slides
 
print STDOUT "\n--- Processing $total slides ---\n"; 

## generate the header table of content of the presentation
## which is also the first page of the talk
&openOverview($overview);

## start the slide count so we can number them
$slideCount = 1;


## @table is the array containing each slide with its title
## for each slide to be generated
## we delete each slide and its title when generated
## so that the current slide and its title are always at $table[0] (for the title)
## and $table[1] (for the slide content)
do {

    ## get rid of the first element contained by the raw presentation array
    shift(@table);
    ## then $table[0] is the title of the slide to be generated
    $table[0] =~ s/\n+/ /g;    ## replace return character by a white space 
    $table[0] =~ s/ +/ /g;     ## concatenate several white spaces to only one
    $table[0] =~ s/^ //;       ## remove all the starting white spaces in the title
    $table[0] =~ s/ $//;       ## remove all trailing white spaces in the title
    ## $slideTitle preserves link(s) in the title
    $slideTitle = $table[0];
    ## need to check if the title contains any anchor
    ## if so it needs to be removed
    ## because the title is being used in the table of content to link to the corresponding slide
    $table[0] =~ s/(.*)<A[^>]*>(.*)<\/A>(.*)/$1$2$3/i;

    ## grab next slide title $table[2] (if there's a next slide)
    ## to be able to use in the 'next' navigation button
    ## keep in mind that $table[1] contains the slide corresponding to the title $table[0]
    $next_slide_title = $table[2] if $table[2];
    ## remove any anchor from the next slide title
    $next_slide_title =~ s/(.*)<A[^>]*>(.*)<\/A>(.*)/$1$2$3/i;

    ## add the title of the current slide to the table of content
    &addTitle("$table[0]",$slideCount);

    ## the current slide content is stored $table[1]
    ## there is an attempt to make sure it's clean HTML
    ## Pierre Fillault's note: use same piece of as used in http://www.w3.org/Web/Tools/CvsCommitScripting
    ## to make use of the validation service
    $slideContent = &clean_html($table[1]) ;

    ## generate the current slide
    ## parameters are:
    ## title of the slide, its content, the slide number, the title of the previous slide and the title of the next slide
    &createSlide("$slideTitle",$slideContent ,$slideCount++,$previous_slide_title,$next_slide_title);

    ## save the title of the previous slide to be displayed in the 'previous' navigation button
    $previous_slide_title="$table[0]";
}
## process the next slide 
while (shift(@table));

## close the table of content
&closeOverview;

## generate more toc with the all the style sheets
## as there's no way of loading a style sheet
## except dynamically, but that would be slow
## and would not work on all platforms (ie would fail on Joe's laptop)
&generateTOC;


print STDOUT "--- Finished ---\n";
exit 0;
##
## end of the slidemaker main program
##############################################################################


##############################################################################
## generate the header of the table of content

sub openOverview 
{
    ## open the file to write to
    open(FOO, ">$_[0].html");

    ## the style sheet used in the table of content is
    ## here is the standard style sheet
    $stylelink = "<link href=\"$cssFile\" rel=\"stylesheet\" type=\"text/css\" title=\"WAM Talk\" />";

    print FOO <<END;
$doctype
<html lang="$baselang">
<head>
<meta http-equiv="Content-type" content="text/html; charset=$charset" />
<title>$talkTitle - Slide list</title>
$stylelink
</head>
$body
END

print FOO <<END;
<h1 class="slideList">$talkTitle</h1>
<p><i>$author</i></p>

END
    if ($baselang == "fr") {
        print FOO <<END;
<div class="slidebody">
<h2>Sommaire</h2>

END
    } else {
        print FOO <<END;
<h2>Table of contents</h2>
END
    }
print FOO <<END;
<ul>
END
}
##
## the beginning of the table of content has been generated and saved
##############################################################################

##############################################################################
## generate the footer of the table of content

sub closeOverview
{
	print FOO <<END;
</ul>
</div>

</body>
</html>
END
    close(FOO); 
}
##
## the toc has been completed and saved
##############################################################################

##############################################################################
## add an item in the toc

sub addTitle 
{
    $_[0] =~ s/\r//ig;      # remove the windows CR+LF 
    $_[0] =~ s/<[^>]+>//g;
    if (length $_[0] == 0) {
	return 1;
    }
    # add accesskey for first 9 slides (`1' - `9'), and tabindex for all slides
    if ($_[1] < 10) {
    print FOO <<END;
<li><a accesskey="$_[1]" tabindex="$_[1]" href="slide$_[1].html">$_[0]</a></li>
END
    } else {
    print FOO <<END;
<li><a tabindex="$_[1]" href="slide$_[1].html">$_[0]</a></li>
END
    }
}
##
##############################################################################

##############################################################################
## generate the current slide

sub createSlide 
{

    # parameters are respectively the slide title, its content,
    # its number, the next slide title and the previous slide title
    if (length $_[0] == 0) {
	return 1;
    }

    # work the slide title, the previous and next titles
    $_[0] =~ s/\r//ig;      # remove the windows CR+LF 
    $_[3] =~ s/\r//ig; 
    $_[4] =~ s/\r//ig;
    $title = $_[0];

    # Remove any tags from the prev & next titles
    $_[3] =~ s/<[^>]+>//g;
    $_[4] =~ s/<[^>]+>//g;
    $title =~ s/<[^>]+>//g;

    # Replace double quotes
#   $_[0] =~ s/"/&#34;/g;
    $_[3] =~ s/"/&#34;/g;
    $_[4] =~ s/"/&#34;/g;

    # work the slide content
    $_[1] =~ s/<\/body>//i; # remove if any
    $_[1] =~ s/<\/html>//i; # remove if any

    $status = sprintf "Slide %2d: %s\n", $_[2], $_[0];
    $status =~ s/<[^>]+>//g;
    print STDOUT $status;

    &verify_html($_[1]);    # check the html

    ## we create only one slides pointing to one css file
     ## write to the slide
    open(SLIDE, ">slide$_[2].html");

    local($navBar);
    local($toclink);
    local($prevlink);
    local($nextlink);

    ## navigation bar with icons to previous slide, toc and next slide
    $navBar = "<p>\n";

    ## the toc link needs to match the slide index
    ## so that the style sheets match between the slides and the toc
    $toclink = "<link rel=\"contents\" href=\"$overview\.html\" title=\"Contents\" />";

    ## initialization of the navigation links
    $nextlink = "";
    $prevlink = "";

    if ($_[2]>1)
      {
	    $prevlink = "<link rel=\"previous\" href=\"slide".eval($_[2]-1).".html\" title=\"Previous\" />";
	    $navBar .= "<a rel=\"previous\" href=\"slide".eval($_[2]-1).".html\" accesskey=\"$prevKey\"><img src=\"$left\" alt=\" previous\" title=\"Back to &#34;$_[3]&#34;\" /></a>";
      }

   	## we're using the first css file, so no need to number the toc
    $navBar = $navBar."<a rel=\"contents\" href=\"$overview\.html\" accesskey=\"$tocKey\"><img src=\"$toc\" alt=\" contents\" title=\"Table of Contents\" /></a>";

    if ($_[2] != $total)
      {
	    $nextlink = "<link rel=\"next\" href=\"slide".eval($_[2]+1).".html\" title=\"Next\" />";
	    $navBar .= "<a rel=\"next\" href=\"slide".eval($_[2]+1).".html\" accesskey=\"$nextKey\"><img src=\"$right\" alt=\" next\" title=\"On to &#34;$_[4]&#34;\" /></a>";
      }
    $navBar .= "</p>";

    $stylelink = "";
    # here is the standard style sheet
    $stylelink .= "<link href=\"$cssFile\" rel=\"stylesheet\" type=\"text/css\" title=\"W3C Talk\" />";

    if ($_[2]>1)
      {
      $body = "<body>";
      }
    else
      {
      $body = "<body class=\"title\">";
      }
    print SLIDE <<END;
$doctype
<html lang="$baselang">
<head>
<meta http-equiv="Content-type" content="text/html; charset=$charset" />
<title>$talkTitle - slide "$title"</title>
$nextlink
$prevlink
$toclink
$stylelink
</head>
$body
<div class="navbar">
END

    if (length $logoFile2 == 0) {
        print SLIDE <<END;
<p class="logo"><a href="$logoLink"><img src="$logoFile" alt="$logoAlt " /></a></p>
END

    } else {
        print SLIDE <<END;
<p class="logo"><a href="$logoLink"><img src="$logoFile" alt="$logoAlt" /></a>&nbsp;
<a href="$logoLink2"><img src="$logoFile2" alt="$logoAlt2" /></a></p>
END
    }
print SLIDE <<END;
$navBar
</div>

<h1 class="slide">$_[0]</h1>

<div class="slidebody">
$_[1]
</div>

<div class="footer">
END

print SLIDE "<p class=\"index\">$_[2]/$total</p>\n";

    print SLIDE <<END;
</div>
</body>
</html>
END

    close(SLIDE);

  return 0;
}

##############################################################################
## generate all the toc of contents needed for each css choosen by the user
## the default toc is not numbered so it can be served by a request to '/'
## (ie it remains Overview.html whereas the other toc are called Overview_#.html)

sub generateTOC {

	## read the general toc
	open(FOO, "<$overview.html"); 
	@TOC = <FOO>;
	close(FOO);
	$toc = "@TOC";

	## starting after the default css
		## create new TOC
		$newTOC = $toc;
		## write to new TOC
		$outfile = $overview."html";
		open(OUT, ">$outfile");
		print OUT $newTOC;
		close(OUT)

}

##############################################################################
# check that the html of the slide 
# is correct (ALT tags, ...)
# This procedure produces only warning
sub verify_html {

    if ($_[0] =~ /<img([^>]*)>/im) {
	if (!($1 =~ /ALT=/im)) {
	    print STDOUT "WARNING: <IMG> without ALT\n";
	    print STDOUT "         <IMG$1>\n" ;
	}
    }
}

##############################################################################
# clean the html of the slide
# remove all <div class="comment">blabla</div>
sub clean_html {
    $_[0] =~ s/<div\s+class\s*=\s*(?:comment[\s>]|\"comment\").*?<\/div>//igs;
    return $_[0];
}
